/*-------------------------------------------------------------------------
  defines.h - global defines

             (c) 2014 Anders Gustafsson <anders.gustafsson@pedago.fi> 

-------------------------------------------------------------------------*/
#define PARMLEN 10	// Max length of parameter or value i hpdisk.cfg

#define SDEBUG	0	// Turns on debug output
					// 0 - none
					// 1 - Just commands
					// 2 - Commands and packet dump
					// 9 - Individual GPIB bytes

//#define SPOLL		// Defined if we support a response to serial poll
//
// Supported drives. Define one for Amigo, one for SS/80
//
#define HP9121D			// HP9121 dual 270K AMIGO floppy drive
//#define HP9122D		// HP9122 dual 630K SS/80 floppy drive
#define HP9134L		// HP9134L 40M SS/80 Winchester drive

#include "integer.h"

#define MLA		0x20		// My listen address = 0 (0+0x20)
#define	MTA		0x40		// My talk address = 0 (0+0x40)
#define	UNL		0x3F		// Unlisten
#define UNT		0x5F		// Untalk
#define	LLO		0x11		// Local lockout
#define	DCL		0x14		// Device clear
#define	PPU		0x15		// Parallell poll unconfigure
#define SPE		0x18		// Serial poll enable
#define	SPD		0x19		// Serial poll disable
#define	GTL		0x01		// Go to local
#define SDC		0x04		// Selected device clear
#define PPC		0x05		// Parallell poll configure
#define GET		0x08		// Group execute trigger
#define TCT		0x09		// Take control

typedef struct {
	char unit;
	char cyl;
	char head;
	char sector;
} DiskType;

typedef struct {		// Controller description, 5 bytes
	BYTE C1;			// Installed unit byte, one unit:
	BYTE C2;			// C1 = 10000000 C2 = 00000001
	BYTE C3;			// Transfer rate in kb/s MSB
	BYTE C4;			// LSB
	BYTE C5;			// Controller type 4 = SS/80 single unit
} ControllerDescriptionType;

typedef struct {		// Unit description, 19 bytes
	BYTE U1;			// Type 0-Fixed, 1-Flexible, 2-Tape (+128-dumb, does not detect media change)
	BYTE U2;			// Device number MSB
	BYTE U3;			// (HP 9133 = 09 13 30)
	BYTE U4;			// LSB
	BYTE U5;			// Bytes per block MSB
	BYTE U6;			// LSB
	BYTE U7;			// Number of buffered blocks
	BYTE U8;			// Burst size (0 for SS/80)
	BYTE U9;			// Block time in ms MSB
	BYTE U10;			// LSB
	BYTE U11;			// Continous average transfer rate MSB
	BYTE U12;			// LSB
	BYTE U13;			// Optimal retry in tens of ms MSB
	BYTE U14;			// LSB
	BYTE U15;			// Access time in tens of ms MSB
	BYTE U16;			// LSB
	BYTE U17;			// Maximum interleave or 0
	BYTE U18;			// Fixed volume byte, one bit per volume, ie 00000111 = 3 volumes
	BYTE U19;			// Removable volume byte, one bit per volume, ie 00000111 = 3 volumes
} UnitDescriptionType;

typedef struct {		// Volume description,  bytes
	BYTE V1;			// Max cylinder MSB
	BYTE V2;			// 
	BYTE V3;			// LSB
	BYTE V4;			// Maximum head
	BYTE V5;			// Maximum sector, MSB
	BYTE V6;			// LSB
	BYTE V7;			// Max value of single vector in blocks MSB
	BYTE V8;
	BYTE V9;
	BYTE V10;
	BYTE V11;
	BYTE V12;			// LSB
	BYTE V13;			// Interleave

} VolumeDescriptionType;

typedef union {
	DWORD Length;
	BYTE Lenbytes[4];
	} LengthType;

#if defined(HP9121D)
#warning Compiled for a HP 9121D Amigo floppy
#define AMIGOID1 0x01
#define AMIGOID2 0x04
#else
#error You must define an Amigo-floppy in defines.h
#endif

#if defined(HP9122D)
#warning Compiled for a HP 9122D SS/80 floppy
#define SS80ID1 0x02
#define SS80ID2 0x22
#elif defined(HP9134L)
#warning Compiled for a HP 9134L 40M SS/80 Winchester drive
#define SS80ID1 0x02
#define SS80ID2 0x21
#else
#error You must define a SS/80 floppy in defines.h
#endif

#define SS80_ERR_READ 	0b00000001
#define SS80_ERR_WRITE 	0b00000010
#define SS80_ERR_SEEK	0b00000100
